<?php

namespace Drupal\user_field_contextual_filter\Plugin\views\argument_default;

use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Drupal\views\Plugin\views\argument_default\ArgumentDefaultPluginBase;

/**
 * Default argument plugin to return a field value from the current user.
 *
 * @ViewsArgumentDefault(
 *   id = "user_field_value",
 *   title = @Translation("Current User Field Value")
 * )
 */
class UserFieldValue extends ArgumentDefaultPluginBase {

    /**
     * Returns the argument value for the contextual filter.
     */
    public function getArgument() {
        $account = $this->currentUser();
        if ($account->isAuthenticated() && !empty($this->options['user_field'])) {
            $user = User::load($account->id());
            $field = $this->options['user_field'];

            if ($user && $user->hasField($field) && !$user->get($field)->isEmpty()) {
                return $user->get($field)->value;
            }
        }
        return NULL;
    }

    /**
     * Builds the options form for the Views UI.
     */
    public function buildOptionsForm(&$form, FormStateInterface $form_state) {
        try {
            $fields = [];
            $definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions('user', 'user');

            foreach ($definitions as $field_name => $definition) {
                if (
                    strpos($field_name, 'field_') === 0 &&
                    $definition->isDisplayConfigurable('view')
                ) {
                    $label = $definition->getLabel() ?: $field_name;
                    $fields[$field_name] = $label . " ($field_name)";
                }
            }

            if (empty($fields)) {
                $fields = ['' => $this->t('No custom user fields available')];
            }

            $form['user_field'] = [
                '#type' => 'select',
                '#title' => $this->t('User field'),
                '#description' => $this->t('Select the field from the current user to use as a contextual argument.'),
                '#options' => $fields,
                '#default_value' => $this->options['user_field'] ?? '',
                '#required' => TRUE,
            ];
        }
        catch (\Throwable $e) {
            \Drupal::logger('user_field_contextual_filter')->error('Error building options form: @msg', ['@msg' => $e->getMessage()]);
            $form['error'] = [
                '#markup' => $this->t('Error: @msg', ['@msg' => $e->getMessage()]),
            ];
        }
    }

    /**
     * Validates the options form.
     */
    public function validateOptionsForm(&$form, FormStateInterface $form_state) {
        // No extra validation needed for now.
    }

    /**
     * Handles submission of the options form.
     */
    public function submitOptionsForm(&$form, FormStateInterface $form_state, &$options = []) {
        $value = $form_state->getValue('user_field');
        if (!empty($value)) {
            $this->options['user_field'] = $value;
        }
    }

    /**
     * Returns the current user.
     */
    protected function currentUser() {
        return \Drupal::currentUser();
    }

}
